/*
    Copyright (C) 2005 - 2009 Manuel Lausch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Manuel Lausch
    Salierstr. 52
    75177 Pforzheim
    GERMANY
    software@manuellausch.de
*/

#include "config.h"
#include "config-player.h"
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <stdlib.h>

using std::string;
using std::endl;
using std::hex;
using std::dec;

config::config()
{
	// Bis jetzt mal nur statische daten spter dann per config file o.

	feldbreite = 50;
	feldhohe = 50;
	quadsize = 12;
	version = "0.5.2";
#ifdef WIN32
    datapath = std::string(getenv("PROGRAMFILES"));
#else
	datapath = std::string(DATADIR);
#endif
	maxplayer = 4;

	player = new config_player[maxplayer];
	player[0].set(50,0x0000ffff, 273, 274, 276, 275);
	player[1].set(50,0xffff00ff, 'w','s','a','d');
	player[2].set(50,0xff00ffff, 'z','h','g','j');
	player[3].set(50,0x00ffffff, '9','o','i','p');

	load();
}

config::~config()
{
	delete[] player;
}

void config::load()
{
// DIe Methode ist ne mittlere katastrophe, aber sie funktioniert erstmal - sofern die passende config fehlerfrei ist
	string file, type, value1;
	unsigned int i = 0,j;
	if((file = get_filepath("snake.conf")) != "\0") // Wenn Datei vorhanden ist dann lade sie auch
	{
		std::ifstream configfile(file.c_str());

		while(configfile >> type)
		{
			if(type == "feldbreite")
			{
				configfile >> feldbreite;
				continue;
			}
			if(type == "feldhohe")
			{
				configfile >> feldhohe;
				continue;
			}
			if(type == "quadsize")
			{
				configfile >> quadsize;
				continue;
			}
			if(type == "datapath")
			{
				configfile >> datapath;
				continue;
			}
			if(type == "maxplayer")
			{
				unsigned int t1, t2, t[4];

				delete[] player;
				configfile >> maxplayer;
				player = new config_player[maxplayer];
				for(i = 0; i < maxplayer; i++)
				{
					configfile >> type;
					configfile >> t1 >> hex >> t2 >> dec;
					for(j = 0; j < 4; j++)
					{
						string map;
						configfile >> map;
						if(map[0] == '\'')
						{
							t[j] = (int)map[1];
						}
						else
						{
							std::istringstream temp(map);
							temp >> t[j];
						}
					}
					player[i].set(t1,t2,t[0],t[1],t[2],t[3]);
					//std::cout << i << " " << type << endl;
				}
				continue;
			}
		}
		configfile.close();
	}
}

void config::save()
{
//Und auch hier ist Win_speziefischer code....
#ifdef WIN32
	std::ofstream configfile(string(string(getenv("HOMEPATH")) + "/snake.conf").c_str()); // configfile immer im homedir des users speichern
#else
	std::ofstream configfile(string(string(getenv("HOME")) + "/snake.conf").c_str()); // configfile immer im homedir des users speichern
#endif

	configfile << "feldbreite " << feldbreite << endl;
	configfile << "feldhohe " << feldhohe << endl;
	configfile << "quadsize " << quadsize << endl;
	configfile << "datapath " << datapath << endl;
	configfile << "maxplayer " << maxplayer << endl;

	unsigned int i = 0;
	for(i = 0; i < maxplayer; i++)
	{
			configfile << "player " << player[i] << endl;
	}
	configfile.close();
}

const unsigned int config::get_feldbreite() const
{
		return feldbreite;
}

const unsigned int config::get_feldhohe() const
{
		return feldhohe;
}

const unsigned int config::get_feldgrose() const
{
	return feldbreite * feldhohe;
}

const std::string & config::get_datapath() const
{
	return datapath;
}

const unsigned int config::get_quadsize() const
{
	return quadsize;
}

void config::set_datapath(std::string pfad)
{
	datapath = pfad;
}

const std::string & config::get_version() const
{
	return version;
}

const unsigned int config::get_maxplayer() const
{
	return maxplayer;
}

config_player *config::get_player(int i)
{
	return &player[i];
}

std::string config::get_filepath(std::string datei)
{
	std::ifstream test;

	// TODO: mehrere return Anweisungen in einer Funktion sind bse!
	// Wer sagt das...?

	// Ist gewnschte Datei im Distri Standardpfad?
	test.open(std::string(datapath + "/snake/" + datei).c_str());
	if(test.is_open())
	{

		test.close();
		return datapath + "/snake/" + datei;
	}

	// überprüfe ob benötigte Datei im aktuellen Verzeichnis ist (cwd)
	test.open(std::string("./" + datei).c_str());
	if(test.is_open())
	{
		test.close();
		return "./" + datei;
	}

	// überprüfe ob benötigte Datei im Unterverzeichnis "res" des aktuellen Verzeichnisses ist
	test.open(std::string("./res/" + datei).c_str());
	if(test.is_open())
	{
		test.close();
		return "./res/" + datei;
	}

	// berprfe ob bentigte Datei im Unterverzeichnis "src/res" des aktuellen Verzeichnisses ist
	test.open(std::string("./src/res/" + datei).c_str());
	if(test.is_open())
	{
		test.close();
		return "./src/res/" + datei;
	}

	// Im Homeverzeichnis suchen
    // Und auch hier ist Win_speziefischer code....
#ifdef WIN32
    test.open(std::string(std::string(getenv("HOMEPATH")) + "/" + datei).c_str());
#else
	test.open(std::string(std::string(getenv("HOME")) + "/" + datei).c_str());
#endif
	if(test.is_open())
	{
		test.close();
#ifdef WIN32
        return std::string(getenv("HOMEPATH")) + "/" + datei;
#else
        return std::string(getenv("HOME")) + "/" + datei;
#endif

	}

	std::cerr << "Fehler: Datei " << datei << " konnte nicht geffnet werden." << std::endl;
	return "\0";

}
